@@ -116,6 +116,7 @@ main(int argc, char *argv[]){
     config.once = false;
 
     config.pidfile = NULL;
+    config.pidfile_set = 0;
     config.program = NULL;
     config.host = NULL;
     config.port = NULL;
@@ -250,6 +251,7 @@ main(int argc, char *argv[]){
     MsgRundown();
     if (config.host) free(config.host);
     if (config.port) free(config.port);
+    if (config.pidfile && !config.pidfile_set) free(config.pidfile);
     free(config.user);
     free(config.password);
     free(config.schema);
