@@ -357,6 +357,8 @@ int KsmCollectionInit(KSM_PARCOLL* data)
     data->regdelay = KSM_PAR_REGDELAY;
     data->pub_safety = KSM_PAR_PUBSAFETY;
     data->ret_safety = KSM_PAR_RETSAFETY;
+    data->rfc5011 = KSM_PAR_RFC5011;
+    data->revoke = KSM_PAR_REVOKE;
 
     return(0);
 }
@@ -467,6 +469,12 @@ int KsmParameterCollection(KSM_PARCOLL* data, int policy_id)
     status = KsmParameterValue(KSM_PAR_DSTTL_STRING, KSM_PAR_DSTTL_CAT, &(data->dsttl), policy_id, &param_id);
     if (status > 0) return status;
 
+    status = KsmParameterValue(KSM_PAR_RFC5011_STRING, KSM_PAR_RFC5011_CAT, &(data->rfc5011), policy_id, &param_id);
+    if (status > 0) return status;
+
+    status = KsmParameterValue(KSM_PAR_REVOKE_STRING, KSM_PAR_REVOKE_CAT, &(data->revoke), policy_id, &param_id);
+    if (status > 0) return status;
+
 /* For now we only set our default KSK rollover scheme */
 /*    status = KsmParameterValue(KSM_PAR_KSK_ROLL_STRING, KSM_PAR_KSK_ROLL_CAT, &(data->kskroll), policy_id, &param_id);
     if (status > 0) return status;
