@@ -1,6 +1,4 @@
 /*
- * $Id: ksm_parameter_value.c 2676 2010-01-11 15:31:31Z sion $
- *
  * Copyright (c) 2008-2009 Nominet UK. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -209,6 +207,26 @@ int KsmParameterRetSafety(KSM_PARCOLL* collection)
     return collection->ret_safety;
 }
 
+int KsmParameterRfc5011(KSM_PARCOLL* collection)
+{
+    /* check the argument */
+    if (collection == NULL) {
+        MsgLog(KSM_INVARG, "NULL collection");
+        return -1;
+    }
+    return collection->rfc5011;
+}
+
+int KsmParameterRevoke(KSM_PARCOLL* collection)
+{
+    /* check the argument */
+    if (collection == NULL) {
+        MsgLog(KSM_INVARG, "NULL collection");
+        return -1;
+    }
+    return collection->revoke;
+}
+
 /*
  * Initial publication interval
  *
