@@ -745,11 +745,13 @@ addns_read(void* zone)
     /* did we already store a new zone transfer on disk? */
     if (!z->xfrd->serial_disk_acquired ||
         z->xfrd->serial_disk_acquired <= z->xfrd->serial_xfr_acquired) {
-        lock_basic_unlock(&z->xfrd->serial_lock);
-        lock_basic_unlock(&z->xfrd->rw_lock);
         if (!z->xfrd->serial_disk_acquired) {
+            lock_basic_unlock(&z->xfrd->serial_lock);
+            lock_basic_unlock(&z->xfrd->rw_lock);
             return ODS_STATUS_XFR_NOT_READY;
         }
+        lock_basic_unlock(&z->xfrd->serial_lock);
+        lock_basic_unlock(&z->xfrd->rw_lock);
         /* do a transaction for DNSKEY and NSEC3PARAM */
         adapi_trans_diff(z, 0);
         ods_log_verbose("[%s] no new xfr ready for zone %s", adapter_str,
@@ -760,6 +762,10 @@ addns_read(void* zone)
     xfrfile = ods_build_path(z->name, ".xfrd", 0, 1);
     file = ods_build_path(z->name, ".xfrd.tmp", 0, 1);
     if (!xfrfile || !file) {
+        free(xfrfile);
+        free(file);
+        lock_basic_unlock(&z->xfrd->serial_lock);
+        lock_basic_unlock(&z->xfrd->rw_lock);
         ods_log_error("[%s] unable to build paths to xfrd files", adapter_str);
         return ODS_STATUS_MALLOC_ERR;
     }
@@ -888,6 +894,8 @@ addns_write(void* zone)
     }
     free((void*) axfrfile);
     free((void*) atmpfile);
+    axfrfile = NULL;
+    atmpfile = NULL;
 
     if (z->db->is_initialized) {
         ixfrfile = ods_build_path(z->name, ".ixfr", 0, 1);
