@@ -188,7 +188,7 @@ static void
 engine_stop_cmdhandler(engine_type* engine)
 {
     ods_log_assert(engine);
-    if (!engine->cmdhandler) {
+    if (!engine->cmdhandler || engine->cmdhandler_done) {
         return;
     }
     ods_log_debug("[%s] stop command handler", engine_str);
@@ -203,7 +203,6 @@ engine_stop_cmdhandler(engine_type* engine)
         ods_log_error("[%s] command handler self pipe trick failed, "
             "unclean shutdown", engine_str);
     }
-    return;
 }
 
 
@@ -556,7 +555,7 @@ engine_setup(engine_type* engine)
         (unsigned long) engine->pid);
     /* catch signals */
     signal_set_engine(engine);
-    action.sa_handler = signal_handler;
+    action.sa_handler = (void (*)(int))signal_handler;
     sigfillset(&action.sa_mask);
     action.sa_flags = 0;
     sigaction(SIGTERM, &action, NULL);
@@ -865,6 +864,9 @@ engine_update_zones(engine_type* engine, ods_status zl_changed)
             lock_basic_lock(&zone->zone_lock);
             zone->task = task;
             lock_basic_unlock(&zone->zone_lock);
+            /* TODO: task is reachable from other threads by means of
+             * zone->task. To fix this we need to nest the locks. But
+             * first investigate any possible deadlocks. */
             lock_basic_lock(&engine->taskq->schedule_lock);
             status = schedule_task(engine->taskq, task, 0);
             lock_basic_unlock(&engine->taskq->schedule_lock);
@@ -885,6 +887,7 @@ engine_update_zones(engine_type* engine, ods_status zl_changed)
     }
     lock_basic_unlock(&engine->zonelist->zl_lock);
     if (engine->dnshandler) {
+        ods_log_debug("[%s] forward notify for all zones", engine_str);
         dnshandler_fwd_notify(engine->dnshandler,
             (uint8_t*) ODS_SE_NOTIFY_CMD, strlen(ODS_SE_NOTIFY_CMD));
     } else if (warnings) {
@@ -1068,11 +1071,9 @@ engine_start(const char* cfgfile, int cmdline_verbosity, int daemonize,
         ods_log_verbose("[%s] close hsm", engine_str);
         hsm_close();
     }
-    if (!engine->cmdhandler_done) {
-        engine_stop_xfrhandler(engine);
-        engine_stop_dnshandler(engine);
-        engine_stop_cmdhandler(engine);
-    }
+    engine_stop_cmdhandler(engine);
+    engine_stop_xfrhandler(engine);
+    engine_stop_dnshandler(engine);
 
 earlyexit:
     if (engine && engine->config) {
