@@ -67,6 +67,7 @@ xfrhandler_create(allocator_type* allocator)
     xfrh->packet = NULL;
     xfrh->netio = NULL;
     xfrh->tcp_set = NULL;
+    xfrh->tcp_waiting_first = NULL;
     xfrh->udp_waiting_first = NULL;
     xfrh->udp_waiting_last = NULL;
     xfrh->udp_use_num = 0;
@@ -196,8 +197,9 @@ xfrhandler_handle_dns(netio_type* ATTR_UNUSED(netio),
     }
     xfrhandler = (xfrhandler_type*) handler->user_data;
     ods_log_assert(event_types & NETIO_EVENT_READ);
-    ods_log_debug("[%s] read forwarded dns packet", xfrh_str);
     received = read(xfrhandler->dnshandler.fd, &buf, MAX_PACKET_SIZE);
+    ods_log_debug("[%s] read forwarded dns packet: %d bytes received",
+        xfrh_str, (int) received);
     if (received == -1) {
         ods_log_error("[%s] unable to forward dns packet: %s", xfrh_str,
             strerror(errno));
