@@ -1,6 +1,4 @@
 /*
- * $Id: namedb.c 5467 2011-08-24 06:51:16Z matthijs $
- *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -529,7 +527,7 @@ namedb_add_nsec3_trigger(namedb_type* db, domain_type* domain,
         dstatus = domain_is_delegpt(domain);
         /* If Opt-Out is being used, owner names of unsigned delegations
            MAY be excluded. */
-        if (dstatus == LDNS_RR_TYPE_NS || domain_ent2unsignedns(domain)) {
+        if (dstatus == LDNS_RR_TYPE_NS) {
             return;
         }
     }
@@ -619,7 +617,7 @@ namedb_del_nsec3_trigger(namedb_type* db, domain_type* domain,
         dstatus = domain_is_delegpt(domain);
         /* If Opt-Out is being used, owner names of unsigned delegations
            MAY be excluded. */
-        if (dstatus == LDNS_RR_TYPE_NS || domain_ent2unsignedns(domain)) {
+        if (dstatus == LDNS_RR_TYPE_NS) {
             denial_diff((denial_type*) domain->denial);
             denial = namedb_del_denial(db, domain->denial);
             denial_cleanup(denial);
@@ -918,6 +916,7 @@ namedb_examine(namedb_type* db)
     ldns_rbnode_t* node = LDNS_RBTREE_NULL;
     domain_type* domain = NULL;
     rrset_type* rrset = NULL;
+    int soa_seen = 0;
 /*
     ldns_rr_type dstatus = LDNS_RR_TYPE_FIRST;
     ldns_rr_type delegpt = LDNS_RR_TYPE_FIRST;
@@ -957,6 +956,21 @@ namedb_examine(namedb_type* db)
                 return ODS_STATUS_CONFLICT_ERR;
             }
         }
+        if (!soa_seen && domain->is_apex) {
+            rrset = domain_lookup_rrset(domain, LDNS_RR_TYPE_SOA);
+            if (rrset) {
+                /* Thou shall have one and only one SOA */
+                if (rrset_count_rr_is_added(rrset) != 1) {
+                    log_rrset(domain->dname, rrset->rrtype,
+                        "Wrong number of SOA records, should be 1", LOG_ERR);
+                    return ODS_STATUS_CONFLICT_ERR;
+                }
+            } else {
+                log_rrset(domain->dname, LDNS_RR_TYPE_SOA, "missing SOA RRset",
+                    LOG_ERR);
+                return ODS_STATUS_CONFLICT_ERR;
+            }
+        }
 /*
         dstatus = domain_is_occluded(domain);
         delegpt = domain_is_delegpt(domain);
