@@ -1,6 +1,4 @@
 /*
- * $Id: axfr.c 4958 2011-04-18 07:11:09Z matthijs $
- *
  * Copyright (c) 2011 NLNet Labs. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -110,8 +108,8 @@ soa_request(query_type* q, engine_type* engine)
         expire = q->zone->xfrd->serial_xfr_acquired;
         expire += ldns_rdf2native_int32(ldns_rr_rdf(rr, SE_SOA_RDATA_EXPIRE));
         if (expire < time_now()) {
-            ods_log_warning("[%s] zone %s expired, not serving soa",
-                axfr_str, q->zone->name);
+            ods_log_warning("[%s] zone %s expired at %u, and it is now %u: "
+                "not serving soa", axfr_str, q->zone->name, expire, time_now());
             ldns_rr_free(rr);
             buffer_pkt_set_rcode(q->buffer, LDNS_RCODE_SERVFAIL);
             ods_fclose(fd);
