@@ -75,8 +75,16 @@ notify_set_timer(notify_type* notify, time_t t)
     if(t > notify_time(notify) + 10) {
         time_t extra = t - notify_time(notify);
         time_t base = extra*9/10;
+#ifdef HAVE_ARC4RANDOM_UNIFORM
+        t = notify_time(notify) + base +
+            arc4random_uniform(extra-base);
+#elif HAVE_ARC4RANDOM
+        t = notify_time(notify) + base +
+            arc4random()%(extra-base);
+#else
         t = notify_time(notify) + base +
             random()%(extra-base);
+#endif
     }
     notify->handler.timeout = &notify->timeout;
     notify->timeout.tv_sec = t;
@@ -274,22 +282,24 @@ notify_handle_reply(notify_type* notify)
     ods_log_assert(xfrhandler);
     ods_log_assert(zone);
     ods_log_assert(zone->name);
-    if ((buffer_pkt_opcode(xfrhandler->packet) != LDNS_PACKET_NOTIFY) ||
+    if (xfrhandler->packet->limit < 3 ||
+        (buffer_pkt_opcode(xfrhandler->packet) != LDNS_PACKET_NOTIFY) ||
         (buffer_pkt_qr(xfrhandler->packet) == 0)) {
-        ods_log_error("[%s] zone %s received bad notify reply opcode/qr",
-            notify_str, zone->name);
+        ods_log_error("[%s] zone %s received bad notify reply opcode/qr from %s",
+            notify_str, zone->name, notify->secondary->address);
         return 0;
     }
     if (buffer_pkt_id(xfrhandler->packet) != notify->query_id) {
-        ods_log_error("[%s] zone %s received bad notify reply id",
-            notify_str, zone->name);
+        ods_log_error("[%s] zone %s received bad notify reply id from %s",
+            notify_str, zone->name, notify->secondary->address);
         return 0;
     }
     /* could check tsig */
     if (buffer_pkt_rcode(xfrhandler->packet) != LDNS_RCODE_NOERROR) {
         const char* str = buffer_rcode2str(buffer_pkt_rcode(xfrhandler->packet));
-        ods_log_error("[%s] zone %s received bad notify rcode %s",
-            notify_str, zone->name, str?str:"UNKNOWN" );
+        ods_log_error("[%s] zone %s received bad notify rcode %s from %s",
+            notify_str, zone->name, str?str:"UNKNOWN",
+            notify->secondary->address);
         if (buffer_pkt_rcode(xfrhandler->packet) != LDNS_RCODE_NOTIMPL) {
             return 1;
         }
