@@ -70,7 +70,7 @@ static xfrd_pkt_status xfrd_handle_packet(xfrd_type* xfrd,
 
 static void xfrd_tcp_obtain(xfrd_type* xfrd, tcp_set_type* set);
 static void xfrd_tcp_read(xfrd_type* xfrd, tcp_set_type* set);
-static void xfrd_tcp_release(xfrd_type* xfrd, tcp_set_type* set);
+static void xfrd_tcp_release(xfrd_type* xfrd, tcp_set_type* set, int open_waiting);
 static void xfrd_tcp_write(xfrd_type* xfrd, tcp_set_type* set);
 static void xfrd_tcp_xfr(xfrd_type* xfrd, tcp_set_type* set);
 static int xfrd_tcp_open(xfrd_type* xfrd, tcp_set_type* set);
@@ -189,11 +189,12 @@ xfrd_recover(xfrd_type* xfrd)
     const char* soa_mname = NULL;
     const char* soa_rname = NULL;
 
-    if (zone && zone->name) {
+    if (zone && zone->name && zone->db &&
+        zone->db->is_initialized && zone->db->have_serial) {
         file = ods_build_path(zone->name, ".xfrd-state", 0, 1);
         if (file) {
-            ods_log_debug("[%s] recover state file zone %s", xfrd_str,
-                zone->name);
+            ods_log_verbose("[%s] recover xfrd.state file %s zone %s", xfrd_str,
+                file, zone->name);
             fd = ods_fopen(file, NULL, "r");
             if (fd) {
                 if (!backup_read_check_str(fd, ODS_SE_FILE_MAGIC_V3)) {
@@ -264,12 +265,12 @@ xfrd_recover(xfrd_type* xfrd)
                 xfrd->timeout.tv_sec = timeout;
                 xfrd->timeout.tv_nsec = 0;
                 xfrd->master = NULL; /* acl_find_num(...) */
-                xfrd->soa.ttl = htonl(soa_ttl);
-                xfrd->soa.serial = htonl(soa_serial);
-                xfrd->soa.refresh = htonl(soa_refresh);
-                xfrd->soa.retry = htonl(soa_retry);
-                xfrd->soa.expire = htonl(soa_expire);
-                xfrd->soa.minimum = htonl(soa_minimum);
+                xfrd->soa.ttl = soa_ttl;
+                xfrd->soa.serial = soa_serial;
+                xfrd->soa.refresh = soa_refresh;
+                xfrd->soa.retry = soa_retry;
+                xfrd->soa.expire = soa_expire;
+                xfrd->soa.minimum = soa_minimum;
                 xfrd->soa.mname[0] = xfrd_recover_dname(xfrd->soa.mname+1,
                     soa_mname);
                 xfrd->soa.rname[0] = xfrd_recover_dname(xfrd->soa.rname+1,
@@ -299,6 +300,9 @@ xfrd_recover_error:
             }
             free(file);
         }
+    } else {
+        ods_log_verbose("[%s] did not recover xfrd.state file zone %s", xfrd_str,
+            (zone && zone->name)?zone->name:"(null)");
     }
     return;
 }
@@ -347,6 +351,7 @@ xfrd_create(void* xfrhandler, void* zone)
     xfrd->serial_xfr_acquired = 0;
     xfrd->serial_disk_acquired = 0;
     xfrd->serial_notify_acquired = 0;
+    xfrd->serial_retransfer = 0;
     lock_basic_unlock(&xfrd->serial_lock);
     xfrd->query_id = 0;
     xfrd->msg_seq_nr = 0;
@@ -354,6 +359,7 @@ xfrd_create(void* xfrhandler, void* zone)
     xfrd->msg_old_serial = 0;
     xfrd->msg_new_serial = 0;
     xfrd->msg_is_ixfr = 0;
+    xfrd->msg_do_retransfer = 0;
     xfrd->udp_waiting = 0;
     xfrd->udp_waiting_next = NULL;
     xfrd->tcp_waiting = 0;
@@ -414,8 +420,16 @@ xfrd_set_timer(xfrd_type* xfrd, time_t t)
     if(t > xfrd_time(xfrd) + 10) {
         time_t extra = t - xfrd_time(xfrd);
         time_t base = extra*9/10;
+#ifdef HAVE_ARC4RANDOM_UNIFORM
+        t = xfrd_time(xfrd) + base +
+            arc4random_uniform(extra-base);
+#elif HAVE_ARC4RANDOM
+        t = xfrd_time(xfrd) + base +
+            arc4random()%(extra-base);
+#else
         t = xfrd_time(xfrd) + base +
             random()%(extra-base);
+#endif
     }
     xfrd->handler.timeout = &xfrd->timeout;
     xfrd->timeout.tv_sec = t;
@@ -664,6 +678,7 @@ xfrd_commit_packet(xfrd_type* xfrd)
     zone_type* zone = NULL;
     char* xfrfile = NULL;
     FILE* fd = NULL;
+    time_t serial_disk_acq = 0;
     ods_log_assert(xfrd);
     zone = (zone_type*) xfrd->zone;
     xfrfile = ods_build_path(zone->name, ".xfrd", 0, 1);
@@ -693,10 +708,16 @@ xfrd_commit_packet(xfrd_type* xfrd)
     }
     /* update soa serial management */
     xfrd->serial_disk = xfrd->msg_new_serial;
+    serial_disk_acq = xfrd->serial_disk_acquired;
     xfrd->serial_disk_acquired = xfrd_time(xfrd);
+    /* ensure newer time */
+    if (xfrd->serial_disk_acquired == serial_disk_acq) {
+        xfrd->serial_disk_acquired++;
+    }
     xfrd->soa.serial = xfrd->serial_disk;
-    if (util_serial_gt(xfrd->serial_disk, xfrd->serial_xfr) &&
-            xfrd->serial_disk_acquired > xfrd->serial_xfr_acquired) {
+    if (xfrd->msg_do_retransfer ||
+            (util_serial_gt(xfrd->serial_disk, xfrd->serial_xfr) &&
+             xfrd->serial_disk_acquired > xfrd->serial_xfr_acquired)) {
         /* reschedule task */
         int ret = 0;
         xfrhandler_type* xfrhandler = (xfrhandler_type*) xfrd->xfrhandler;
@@ -716,6 +737,9 @@ xfrd_commit_packet(xfrd_type* xfrd)
             engine_wakeup_workers(engine);
         }
     }
+    /* reset retransfer */
+    xfrd->msg_do_retransfer = 0;
+
     lock_basic_unlock(&xfrd->serial_lock);
     lock_basic_unlock(&xfrd->rw_lock);
     lock_basic_unlock(&zone->zone_lock);
@@ -755,7 +779,11 @@ xfrd_dump_packet(xfrd_type* xfrd, buffer_type* buffer)
         return;
     }
     lock_basic_lock(&xfrd->rw_lock);
-    fd = ods_fopen(xfrfile, NULL, "a");
+    if (xfrd->msg_do_retransfer && !xfrd->msg_seq_nr && !xfrd->msg_is_ixfr) {
+        fd = ods_fopen(xfrfile, NULL, "w");
+    } else {
+        fd = ods_fopen(xfrfile, NULL, "a");
+    }
     free((void*) xfrfile);
     if (!fd) {
         ods_log_crit("[%s] unable to dump packet zone %s: ods_fopen() failed "
@@ -968,7 +996,7 @@ xfrd_parse_rrs(xfrd_type* xfrd, buffer_type* buffer, uint16_t count,
                      /* got IXFR but need AXFR */
                      return ODS_STATUS_REQAXFR;
                  }
-                 if (serial != xfrd->serial_disk) {
+                 if (!xfrd->msg_do_retransfer && serial != xfrd->serial_disk) {
                      lock_basic_unlock(&xfrd->serial_lock);
                      /* bad start serial in IXFR */
                      return ODS_STATUS_INSERIAL;
@@ -1072,7 +1100,7 @@ xfrd_parse_packet(xfrd_type* xfrd, buffer_type* buffer)
     ancount = buffer_pkt_ancount(buffer);
     if (xfrd->msg_rr_count == 0 && ancount == 0) {
         if (xfrd->tcp_conn == -1 && buffer_pkt_tc(buffer)) {
-            ods_log_debug("[%s] zone %s received tc from %s, retry tcp",
+            ods_log_info("[%s] zone %s received tc from %s, retry tcp",
                 xfrd_str, zone->name, xfrd->master->address);
             return XFRD_PKT_TC;
         }
@@ -1093,8 +1121,9 @@ xfrd_parse_packet(xfrd_type* xfrd, buffer_type* buffer)
         }
         /* check serial */
         lock_basic_lock(&xfrd->serial_lock);
-        if (xfrd->serial_disk_acquired && xfrd->serial_disk == serial) {
-            ods_log_debug("[%s] zone %s got update indicating current "
+        if (!xfrd->msg_do_retransfer &&
+            xfrd->serial_disk_acquired && xfrd->serial_disk == serial) {
+            ods_log_info("[%s] zone %s got update indicating current "
                 "serial %u from %s", xfrd_str, zone->name, serial,
                  xfrd->master->address);
             xfrd->serial_disk_acquired = xfrd_time(xfrd);
@@ -1116,9 +1145,9 @@ xfrd_parse_packet(xfrd_type* xfrd, buffer_type* buffer)
                 return XFRD_PKT_BAD;
             }
         }
-        if (xfrd->serial_disk_acquired &&
+        if (!xfrd->msg_do_retransfer && xfrd->serial_disk_acquired &&
             !util_serial_gt(serial, xfrd->serial_disk)) {
-            ods_log_debug("[%s] zone %s ignoring old serial %u from %s "
+            ods_log_info("[%s] zone %s ignoring old serial %u from %s "
                 "(have %u)", xfrd_str, zone->name, serial,
                 xfrd->master->address, xfrd->serial_disk);
             lock_basic_unlock(&xfrd->serial_lock);
@@ -1126,7 +1155,7 @@ xfrd_parse_packet(xfrd_type* xfrd, buffer_type* buffer)
         }
 
         xfrd->msg_new_serial = serial;
-        if (xfrd->serial_disk_acquired) {
+        if (!xfrd->msg_do_retransfer && xfrd->serial_disk_acquired) {
             xfrd->msg_old_serial = xfrd->serial_disk;
         } else {
             xfrd->msg_old_serial = 0;
@@ -1143,13 +1172,13 @@ xfrd_parse_packet(xfrd_type* xfrd, buffer_type* buffer)
     }
     /* check tc bit */
     if (xfrd->tcp_conn == -1 && buffer_pkt_tc(buffer)) {
-        ods_log_debug("[%s] zone %s received tc from %s, retry tcp",
+        ods_log_info("[%s] zone %s received tc from %s, retry tcp",
             xfrd_str, zone->name, xfrd->master->address);
         return XFRD_PKT_TC;
     }
     if (xfrd->tcp_conn == -1 && ancount < 2) {
         /* too short to be a real ixfr/axfr data transfer */
-        ods_log_debug("[%s] zone %s received too short udp reply from %s, "
+        ods_log_info("[%s] zone %s received too short udp reply from %s, "
             "retry tcp", xfrd_str, zone->name, xfrd->master->address);
         return XFRD_PKT_TC;
     }
@@ -1189,7 +1218,8 @@ xfrd_handle_packet(xfrd_type* xfrd, buffer_type* buffer)
     ods_log_assert(zone);
     ods_log_assert(zone->name);
     res = xfrd_parse_packet(xfrd, buffer);
-    ods_log_debug("[%s] zone %s xfr packet parsed (res %d)", xfrd_str, zone->name, res);
+    ods_log_debug("[%s] zone %s xfr packet parsed (res %d)", xfrd_str,
+        zone->name, res);
 
     switch (res) {
         case XFRD_PKT_MORE:
@@ -1229,14 +1259,14 @@ xfrd_handle_packet(xfrd_type* xfrd, buffer_type* buffer)
     /* next time */
     lock_basic_lock(&xfrd->serial_lock);
 
-    ods_log_debug("[%s] zone %s notify acquired %u, serial on disk %u, "
-        "notify serial %u", xfrd_str, zone->name,
+    ods_log_info("[%s] zone %s transfer done [notify acquired %u, serial on "
+        "disk %u, notify serial %u]", xfrd_str, zone->name,
         xfrd->serial_notify_acquired, xfrd->serial_disk,
         xfrd->serial_notify);
 
     if (xfrd->serial_notify_acquired &&
         !util_serial_gt(xfrd->serial_notify, xfrd->serial_disk)) {
-        ods_log_debug("[%s] zone %s reset notify acquired", xfrd_str,
+        ods_log_verbose("[%s] zone %s reset notify acquired", xfrd_str,
             zone->name);
         xfrd->serial_notify_acquired = 0;
     }
@@ -1249,7 +1279,7 @@ xfrd_handle_packet(xfrd_type* xfrd, buffer_type* buffer)
     }
     lock_basic_unlock(&xfrd->serial_lock);
     /* try to get an even newer serial */
-    ods_log_debug("[%s] zone %s get newer serial", xfrd_str, zone->name);
+    ods_log_info("[%s] zone %s try get newer serial", xfrd_str, zone->name);
     return XFRD_PKT_BAD;
 }
 
@@ -1293,7 +1323,7 @@ xfrd_tcp_write(xfrd_type* xfrd, tcp_set_type* set)
             ods_log_error("[%s] zone %s cannot tcp connect to %s: %s",
                 xfrd_str, zone->name, xfrd->master->address, strerror(errno));
             xfrd_set_timer_now(xfrd);
-            xfrd_tcp_release(xfrd, set);
+            xfrd_tcp_release(xfrd, set, 1);
             return;
         }
     }
@@ -1302,7 +1332,7 @@ xfrd_tcp_write(xfrd_type* xfrd, tcp_set_type* set)
         ods_log_error("[%s] zone %s cannot tcp write to %s: %s",
             xfrd_str, zone->name, xfrd->master->address, strerror(errno));
         xfrd_set_timer_now(xfrd);
-        xfrd_tcp_release(xfrd, set);
+        xfrd_tcp_release(xfrd, set, 1);
         return;
     }
     if (ret == 0) {
@@ -1357,14 +1387,14 @@ xfrd_tcp_open(xfrd_type* xfrd, tcp_set_type* set)
         ods_log_error("[%s] zone %s cannot create tcp socket to %s: %s",
             xfrd_str, zone->name, xfrd->master->address, strerror(errno));
         xfrd_set_timer_now(xfrd);
-        xfrd_tcp_release(xfrd, set);
+        xfrd_tcp_release(xfrd, set, 0);
         return 0;
     }
     if (fcntl(fd, F_SETFL, O_NONBLOCK) == -1) {
         ods_log_error("[%s] zone %s cannot fcntl tcp socket to %s: %s",
             xfrd_str, zone->name, xfrd->master->address, strerror(errno));
         xfrd_set_timer_now(xfrd);
-        xfrd_tcp_release(xfrd, set);
+        xfrd_tcp_release(xfrd, set, 0);
         return 0;
     }
     to_len = xfrd_acl_sockaddr_to(xfrd->master, &to);
@@ -1375,7 +1405,7 @@ xfrd_tcp_open(xfrd_type* xfrd, tcp_set_type* set)
         ods_log_error("[%s] zone %s cannot connect tcp socket to %s: %s",
             xfrd_str, zone->name, xfrd->master->address, strerror(errno));
         xfrd_set_timer_now(xfrd);
-        xfrd_tcp_release(xfrd, set);
+        xfrd_tcp_release(xfrd, set, 0);
         return 0;
     }
     xfrd->handler.fd = fd;
@@ -1392,6 +1422,7 @@ xfrd_tcp_open(xfrd_type* xfrd, tcp_set_type* set)
 static void
 xfrd_tcp_obtain(xfrd_type* xfrd, tcp_set_type* set)
 {
+    xfrhandler_type* xfrhandler;
     int i = 0;
 
     ods_log_assert(set);
@@ -1425,7 +1456,11 @@ xfrd_tcp_obtain(xfrd_type* xfrd, tcp_set_type* set)
         xfrd_str, TCPSET_MAX);
     xfrd->tcp_waiting = 1;
     xfrd_unset_timer(xfrd);
-    return;
+
+    /* add it to the waiting queue */
+    xfrhandler = (xfrhandler_type*) xfrd->xfrhandler;
+    xfrd->tcp_waiting_next = xfrhandler->tcp_waiting_first;
+    xfrhandler->tcp_waiting_first = xfrd;
 }
 
 
@@ -1451,14 +1486,15 @@ xfrd_tcp_xfr(xfrd_type* xfrd, tcp_set_type* set)
     /* start AXFR or IXFR for the zone */
     tcp = set->tcp_conn[xfrd->tcp_conn];
 
-    if (xfrd->serial_xfr_acquired <= 0 || xfrd->master->ixfr_disabled) {
-        ods_log_debug("[%s] zone %s request axfr to %s", xfrd_str,
+    if (xfrd->msg_do_retransfer || xfrd->serial_xfr_acquired <= 0 ||
+        xfrd->master->ixfr_disabled) {
+        ods_log_info("[%s] zone %s request axfr to %s", xfrd_str,
             zone->name, xfrd->master->address);
         buffer_pkt_query(tcp->packet, zone->apex, LDNS_RR_TYPE_AXFR,
             zone->klass);
     } else {
-        ods_log_debug("[%s] zone %s request ixfr to %s", xfrd_str,
-            zone->name, xfrd->master->address);
+        ods_log_info("[%s] zone %s request tcp/ixfr=%u to %s", xfrd_str,
+            zone->name, xfrd->soa.serial, xfrd->master->address);
         buffer_pkt_query(tcp->packet, zone->apex, LDNS_RR_TYPE_IXFR,
             zone->klass);
         buffer_pkt_set_nscount(tcp->packet, 1);
@@ -1474,7 +1510,7 @@ xfrd_tcp_xfr(xfrd_type* xfrd, tcp_set_type* set)
     xfrd_tsig_sign(xfrd, tcp->packet);
     buffer_flip(tcp->packet);
     tcp->msglen = buffer_limit(tcp->packet);
-    ods_log_debug("[%s] zone %s sending tcp query id=%d", xfrd_str,
+    ods_log_verbose("[%s] zone %s sending tcp query id=%d", xfrd_str,
         zone->name, xfrd->query_id);
     /* wait for select to complete connect before write */
     return;
@@ -1498,7 +1534,7 @@ xfrd_tcp_read(xfrd_type* xfrd, tcp_set_type* set)
     ret = tcp_conn_read(tcp);
     if (ret == -1) {
         xfrd_set_timer_now(xfrd);
-        xfrd_tcp_release(xfrd, set);
+        xfrd_tcp_release(xfrd, set, 1);
         return;
     }
     if (ret == 0) {
@@ -1513,21 +1549,21 @@ xfrd_tcp_read(xfrd_type* xfrd, tcp_set_type* set)
             break;
         case XFRD_PKT_XFR:
         case XFRD_PKT_NEWLEASE:
-            ods_log_debug("[%s] tcp read %s: release connection", xfrd_str,
+            ods_log_verbose("[%s] tcp read %s: release connection", xfrd_str,
                 XFRD_PKT_XFR?"xfr":"newlease");
-            xfrd_tcp_release(xfrd, set);
+            xfrd_tcp_release(xfrd, set, 1);
             ods_log_assert(xfrd->round_num == -1);
             break;
         case XFRD_PKT_NOTIMPL:
             xfrd->master->ixfr_disabled = time_now();
-            ods_log_debug("[%s] disable ixfr requests for %s from now (%u)",
+            ods_log_verbose("[%s] disable ixfr requests for %s from now (%u)",
                 xfrd_str, xfrd->master->address, xfrd->master->ixfr_disabled);
             /* break; */
         case XFRD_PKT_BAD:
         default:
             ods_log_debug("[%s] tcp read %s: release connection", xfrd_str,
                 ret==XFRD_PKT_BAD?"bad":"notimpl");
-            xfrd_tcp_release(xfrd, set);
+            xfrd_tcp_release(xfrd, set, 1);
             xfrd_make_request(xfrd);
             break;
     }
@@ -1536,12 +1572,14 @@ xfrd_tcp_read(xfrd_type* xfrd, tcp_set_type* set)
 
 
 /**
- * Release tcp.
- *
+ * Release tcp connection from set for xfrd. If there are waiting TCP
+ * connections open as many as free slots in set. This step is skipped
+ * if open_waiting flag is unset.
  */
 static void
-xfrd_tcp_release(xfrd_type* xfrd, tcp_set_type* set)
+xfrd_tcp_release(xfrd_type* xfrd, tcp_set_type* set, int open_waiting)
 {
+    xfrhandler_type* xfrhandler;
     int conn = 0;
     zone_type* zone = NULL;
 
@@ -1565,7 +1603,37 @@ xfrd_tcp_release(xfrd_type* xfrd, tcp_set_type* set)
     }
     set->tcp_conn[conn]->fd = -1;
     set->tcp_count --;
-    return;
+
+    /* see if there are any connections waiting for a slot. Or return. */
+    if (!open_waiting) return;
+    xfrhandler = (xfrhandler_type*) xfrd->xfrhandler;
+    while (xfrhandler->tcp_waiting_first && set->tcp_count < TCPSET_MAX) {
+        int i;
+        xfrd_type* waiting_xfrd = xfrhandler->tcp_waiting_first;
+        xfrhandler->tcp_waiting_first = waiting_xfrd->tcp_waiting_next;
+        waiting_xfrd->tcp_waiting_next = NULL;
+
+        /* find a free tcp_buffer */
+        for (i=0; i < TCPSET_MAX; i++) {
+            if (set->tcp_conn[i]->fd == -1) {
+                waiting_xfrd->tcp_conn = i;
+                set->tcp_count++;
+                break;
+            }
+        }
+        waiting_xfrd->tcp_waiting = 0;
+        /* stop udp use (if any) */
+        if (waiting_xfrd->handler.fd != -1) {
+            xfrd_udp_release(waiting_xfrd);
+        }
+        /* if xfrd_tcp_open() fails its slot in set->tcp_conn[]
+         * is released. Continue to next. We don't put it back in the
+         * waiting queue, it would keep the signer busy retrying, making
+         * things only worse. */
+        if (xfrd_tcp_open(waiting_xfrd, set)) {
+            xfrd_tcp_xfr(waiting_xfrd, set);
+        }
+    }
 }
 
 
@@ -1658,8 +1726,8 @@ xfrd_udp_send_request_ixfr(xfrd_type* xfrd)
     xfrd_tsig_sign(xfrd, xfrhandler->packet);
     buffer_flip(xfrhandler->packet);
     xfrd_set_timer(xfrd, xfrd_time(xfrd) + XFRD_UDP_TIMEOUT);
-    ods_log_debug("[%s] zone %s sending udp query id=%d qtype=IXFR to %s",
-        xfrd_str, zone->name, xfrd->query_id, xfrd->master->address);
+    ods_log_info("[%s] zone %s request udp/ixfr=%u to %s", xfrd_str,
+        zone->name, xfrd->soa.serial, xfrd->master->address);
     if((fd = xfrd_udp_send(xfrd, xfrhandler->packet)) == -1) {
         return -1;
     }
@@ -1680,7 +1748,7 @@ xfrd_udp_obtain(xfrd_type* xfrd)
     xfrhandler = (void*) xfrd->xfrhandler;
     if (xfrd->tcp_conn != -1) {
         /* no tcp and udp at the same time */
-        xfrd_tcp_release(xfrd, xfrhandler->tcp_set);
+        xfrd_tcp_release(xfrd, xfrhandler->tcp_set, 1);
     }
     if (xfrhandler->udp_use_num < XFRD_MAX_UDP) {
             xfrhandler->udp_use_num++;
@@ -1758,7 +1826,7 @@ xfrd_udp_read(xfrd_type* xfrd)
     res = xfrd_handle_packet(xfrd, xfrhandler->packet);
     switch (res) {
         case XFRD_PKT_TC:
-            ods_log_debug("[%s] truncation from %s",
+            ods_log_verbose("[%s] truncation from %s",
                 xfrd_str, xfrd->master->address);
             xfrd_udp_release(xfrd);
             xfrd_set_timer(xfrd, xfrd_time(xfrd) + XFRD_TCP_TIMEOUT);
@@ -1766,7 +1834,7 @@ xfrd_udp_read(xfrd_type* xfrd)
             break;
         case XFRD_PKT_XFR:
         case XFRD_PKT_NEWLEASE:
-            ods_log_debug("[%s] xfr/newlease from %s",
+            ods_log_verbose("[%s] xfr/newlease from %s",
                 xfrd_str, xfrd->master->address);
             /* nothing more to do */
             ods_log_assert(xfrd->round_num == -1);
@@ -1774,7 +1842,7 @@ xfrd_udp_read(xfrd_type* xfrd)
             break;
         case XFRD_PKT_NOTIMPL:
             xfrd->master->ixfr_disabled = time_now();
-            ods_log_debug("[%s] disable ixfr requests for %s from now (%u)",
+            ods_log_verbose("[%s] disable ixfr requests for %s from now (%u)",
                 xfrd_str, xfrd->master->address, xfrd->master->ixfr_disabled);
             /* break; */
         case XFRD_PKT_BAD:
@@ -1906,16 +1974,27 @@ xfrd_make_request(xfrd_type* xfrd)
         xfrd->master->ixfr_disabled = 0;
     }
     /* perform xfr request */
-    ods_log_debug("[%s] zone %s make request round %d master %s:%u",
-        xfrd_str, zone->name, xfrd->round_num, xfrd->master->address,
-	xfrd->master->port);
-    if (xfrd->serial_xfr_acquired && !xfrd->master->ixfr_disabled) {
+    if (xfrd->serial_xfr_acquired && !xfrd->master->ixfr_disabled &&
+        !xfrd->serial_retransfer) {
         xfrd_set_timer(xfrd, xfrd_time(xfrd) + XFRD_UDP_TIMEOUT);
+
+        ods_log_verbose("[%s] zone %s make request [udp round %d master %s:%u]",
+            xfrd_str, zone->name, xfrd->round_num, xfrd->master->address,
+	    xfrd->master->port);
         xfrd_udp_obtain(xfrd);
-    } else if (!xfrd->serial_xfr_acquired || xfrd->master->ixfr_disabled) {
+    } else if (!xfrd->serial_xfr_acquired || xfrd->master->ixfr_disabled ||
+        xfrd->serial_retransfer) {
         xfrhandler_type* xfrhandler = (xfrhandler_type*) xfrd->xfrhandler;
         ods_log_assert(xfrhandler);
+        if (xfrd->serial_retransfer) {
+            xfrd->msg_do_retransfer = 1;
+            xfrd->serial_retransfer = 0;
+        }
         xfrd_set_timer(xfrd, xfrd_time(xfrd) + XFRD_TCP_TIMEOUT);
+
+        ods_log_verbose("[%s] zone %s make request [tcp round %d master %s:%u]",
+            xfrd_str, zone->name, xfrd->round_num, xfrd->master->address,
+	    xfrd->master->port);
         xfrd_tcp_obtain(xfrd, xfrhandler->tcp_set);
     }
     return;
@@ -1961,7 +2040,7 @@ xfrd_handle_zone(netio_type* ATTR_UNUSED(netio),
            /* tcp connection timed out. Stop it. */
            ods_log_deeebug("[%s] zone %s event tcp timeout", xfrd_str,
                zone->name);
-           xfrd_tcp_release(xfrd, xfrhandler->tcp_set);
+           xfrd_tcp_release(xfrd, xfrhandler->tcp_set, 1);
            /* continue to retry; as if a timeout happened */
            event_types = NETIO_EVENT_TIMEOUT;
         }
@@ -2055,7 +2134,7 @@ xfrd_backup(xfrd_type* xfrd)
                 fprintf(fd, "%s\n", ODS_SE_FILE_MAGIC_V3);
                 fprintf(fd, ";;Zone: name %s ttl %u mname ",
                     zone->name,
-                    (unsigned) ntohl(xfrd->soa.ttl));
+                    (unsigned) xfrd->soa.ttl);
                 xfrd_backup_dname(fd, xfrd->soa.mname),
                 fprintf(fd, " rname ");
                 xfrd_backup_dname(fd, xfrd->soa.rname),
