@@ -105,11 +105,17 @@ struct xfrd_struct
 
     /* soa serial management */
     uint32_t serial_xfr;
+    /* Last received serial via notify. Only valid if
+     * serial_notify_acquired is not 0 */
     uint32_t serial_notify;
+    /* current serial on inbound side */
     uint32_t serial_disk;
     time_t serial_xfr_acquired;
+    /* time of last received notify that is being handled. If non-zero
+     * it indicates a transfer is in progress */
     time_t serial_notify_acquired;
     time_t serial_disk_acquired;
+    uint8_t serial_retransfer;
     soa_type soa;
 
     /* timeout and event handling */
@@ -123,6 +129,7 @@ struct xfrd_struct
     uint32_t msg_new_serial;
     size_t msg_rr_count;
     uint8_t msg_is_ixfr;
+    uint8_t msg_do_retransfer;
     tsig_rr_type* tsig_rr;
 
     xfrd_type* tcp_waiting_next;
